#!/bin/bash
#
#   USAGE:  build_range.sh <product_dir> [tar]
#   NOTE:   See COMPILER.txt

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`

    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
maketar=${2}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=range
origpkgver=v3_0_4_0
origpkgdotver=`echo ${origpkgver#v3_} | sed -e 's/_/./g'`
pkgtarstem=range-v3-${origpkgdotver}
pkgver=${origpkgver}
ssibuildshims_version=v1_04_12

get_this_dir

get_ssibuildshims

source define_basics NO_QUAL NO_FQ_DIR

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/include ]
then
    ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} NO_FLAVOR
    exit 0
fi

echo "building ${package} ${pkgver}"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------
mkdir -p ${blddir}/src
if [ ! -d ${blddir}/src ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi

cd ${blddir}/src || ssi_die "cannot cd to ${blddir}/src"
tar xf ${tardir}/${pkgtarstem}.tar.gz || ssi_die "Unable to unwind ${tardir}/${pkgtarstem}.tar.gz"
##patch -b -p0 < ${patchdir}/range.patch || ssi_die "application of range.patch failed."

# fake ups declare           
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} build NO_FLAVOR

setup -B ${package} ${pkgver} -q build -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "fake setup failed"

gccprod=$(latest_version -t ${product_dir} --ge 6.1.0 gcc -z "${product_dir}:${PRODUCTS}")
if [[ -n "${gccprod}" ]]; then
  echo "${gccprod} is available for use with tests"
  eval setup -B ${gccprod} -z "${product_dir}:${PRODUCTS}" || \
    ssi_die "Unable to set up ${gccprod}"
  export CXX=g++
  export CC=gcc
elif [[ "${OS1}" == Darwin ]]; then
  echo "Using native cc/c++ for tests"
  export CXX=c++
  export CC=cc
else
  echo "Modern (GCC >= 6.1) compiler is not available."
  ssi_die "range cannot be built without also building the tests"
fi

source setup_cmake  || ssi_die "setup of cmake failed"
ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

ups active

cd ${blddir} || ssi_die "cannot cd to ${blddir}"

# checking g++
${CXX} -xc++ -std=c++11 -c /dev/null >/dev/null 2>&1 && cppdef=11
${CXX} -xc++ -std=c++14 -c /dev/null >/dev/null 2>&1 && cppdef=14
${CXX} -xc++ -std=c++17 -c /dev/null >/dev/null 2>&1 && cppdef=17
echo "build tests with -std=c++${cppdef}"
if [[ -z "${cppdef}" ]]; then
  ssi_die "range tests require C++11 or better."
fi
cmake -DCMAKE_INSTALL_PREFIX="${pkgdir}" -DRANGES_CXX_STD=${cppdef} \
  ${blddir}/src/${pkgtarstem} || ssi_die "cmake step failed"
make -j ${ncore} install || ssi_die "make install failed."
ctest -j ${ncore}

if [ ! -d ${pkgdir}/include ]
then
   echo "ERROR: failed to create ${pkgdir}/include"
   echo "       Something is very wrong"
   exit 1
fi

${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} NO_FLAVOR || \
  ssi_die "Unable to declare product to UPS."

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
unsetup ${package}
setup -B ${package} ${pkgver} -z ${product_dir} || ssi_die "unable to set up newly-built product."

# this must be last
if [ "${maketar}" = "tar" ]
then
    ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} NO_FLAVOR
fi

exit 0
